<?php

namespace Service;

class WebSocketUpdateHandler {

	protected $entityManager;
	protected $KDSManagerService;
	protected $KDSManagerController;
	protected $statistics;
	protected $messageService;
	protected $KDSTransferService;
	protected $orderStash;

	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Service\KDSManager $KDSManagerService,
		\Controller\KDSManager $KDSManagerController,
		\Service\OrdersEngine $OrdersEngine,
		\Service\Statistics $statistics,
		\Service\MessageService $messageService,
		\Service\KDSTransfer $KDSTransferService,
		\Util\OrderStash $orderStash,
		$shouldLog,
		$pos0000196913) {
		$this->entityManager = $entityManager;
		$this->KDSManagerService = $KDSManagerService;
		$this->KDSManagerController = $KDSManagerController;
		$this->OrdersEngine = $OrdersEngine;
		$this->statistics = $statistics;
		$this->messageService = $messageService;
		$this->KDSTransferService = $KDSTransferService;
		$this->orderStash = $orderStash;
		$this->shouldLog = $shouldLog;
		$this->logTimer = array();
        $this->pos0000196913 = $pos0000196913;

        $this->GET_KDSOPERACAOTEMP = $this->pos0000196913 ? "SELECT TOP 5 NRSEQOPERACAOPK, DSOPERACAO FROM KDSOPERACAOTEMP" : "SELECT TOP 5 NRSEQOPERACAO, DSOPERACAO FROM KDSOPERACAOTEMP";
        $this->DEL_KDSOPERACAOTEMP = $this->pos0000196913 ? "DELETE FROM KDSOPERACAOTEMP WHERE NRSEQOPERACAOPK = :NRSEQOPERACAOPK" : "DELETE FROM KDSOPERACAOTEMP WHERE NRSEQOPERACAO = :NRSEQOPERACAO";
	}

	private function logcheckupdate ($event, $space= false){
		if(isset($this->shouldLog) && $this->shouldLog === true){
				try{
					if(!isset($this->logTimer[$event])){
						$this->logTimer[$event] = round(microtime(true) * 1000);
					}else{
						$tempoFinal = round(microtime(true) * 1000);
						$total = $tempoFinal - $this->logTimer[$event];

						//testa se a pasta existe e cria caso não exista
						$folder = substr(__DIR__, 0, 1) . ":\\TEKNISA\\kds\\logs\\";
						if (!file_exists($folder)) {
							mkdir($folder, 0777, true);
						}
						//--grava o log
						$space = $space ? "\n\r\n\r" : "";
						$line = date("d-m-Y H:i:s") . " - Tempo - $event: " . $total . "\r $space";
						$drive = substr(__DIR__, 0, 1);
						$date = date("dmY");
						file_put_contents($drive . ":\\TEKNISA\\kds\\logs\\webSocketUpdateHandler" . $date . ".txt", $line, FILE_APPEND);
						unset($this->logTimer[$event]);
					}
				} catch(\Exception $e){
				    $this->util->logException($e->getMessage());
			    }
		}
    }

	public function handleUpdates($socketClient) {
		$newUpdates = $this->entityManager->getConnection()->fetchAll($this->GET_KDSOPERACAOTEMP);
		foreach ($newUpdates as $currentUpdate) {
			$operation = json_decode($currentUpdate['DSOPERACAO'], true);
			switch ($operation['operation']) {
				case 'playItem':
					$this->playItem($operation['params']);
				break;
				case 'concludeItem':
					$this->concludeItem($operation['params']);
				break;
				case 'expediteOrder':
					$this->expediteOrder($operation['params']);
				break;
				case 'concludeZoomEvent':
					$this->concludeZoomEvent($operation['params']);
				break;
				case 'releaseItem' :
					$this->releaseOrder($operation['params']);
				break;
				case 'cancelItem':
					$this->cancelItem($operation['params']);
				break;
				case 'getDataFromOneClient':
					$this->getDataFromOneClient($operation['params'], $socketClient);
				break;
				case 'clearRequestsScreen':
					$this->clearRequestsScreen($operation['params']);
				break;
				case 'deleteMessage':
					$this->deleteMessage($operation['params']);
				break;
				case 'updateTransferTable':
					$this->updateTransferTable($operation['params']);
				break;
				case 'updateNotificationTime':
					$this->updateNotificationTime($operation['params']);
				break;
				/* case 'refreshOrdemItem':
					$this->refreshOrdemItem($operation['params']);
				break; */
				//updateNotificationTime
				default:
					// operação não reconhecida
				break;
			}
			// apagar registro da KDSOPERACAOTEMP passando NRSEQOPERACAO como parametro
            if ($this->pos0000196913){
                $params = array(
                    ':NRSEQOPERACAOPK' => $currentUpdate['NRSEQOPERACAOPK']
                );  
            } else {
                $params = array(
                    ':NRSEQOPERACAO' => $currentUpdate['NRSEQOPERACAO'] 
                );
            }
            
			$this->entityManager->getConnection()->executeQuery($this->DEL_KDSOPERACAOTEMP, $params);
		}
	}

	private function playItem($params) {
		$this->logcheckupdate("playItem - changeStatusItemRequestPlay");
		$this->KDSManagerService->changeStatusItemRequestPlay($params['CDFILIAL'], $params['NRPEDIDOFOS'], $params['NRITPEDIDOFOS'], $params['CDSETOR']);
		$this->logcheckupdate("playItem - changeStatusItemRequestPlay");
		$this->logcheckupdate("playItem - playItemStatistics");
		$this->statistics->playItemStatistics($params['CDFILIAL'], $params['NRPEDIDOFOS'], $params['NRITPEDIDOFOS'], $params['CDSETOR']);
		$this->logcheckupdate("playItem - playItemStatistics");
		$this->logcheckupdate("playItem - updateDtUltAtuItPedidoFosByNrPedidoFos");
		$this->KDSManagerService->updateDtUltAtuItPedidoFosByNrPedidoFos($params['CDFILIAL'], $params['NRPEDIDOFOS']);
		$this->logcheckupdate("playItem - updateDtUltAtuItPedidoFosByNrPedidoFos", true);
	}

	private function concludeItem($params) {
		$this->logcheckupdate("concludeItem - doConclude");
		$this->KDSManagerService->doConclude($params['CDFILIAL'], $params['NRPEDIDOFOS'], $params['NRITPEDIDOFOS'], $params['IDSITITPEDFOS']);
		$this->logcheckupdate("concludeItem - doConclude");
		$this->logcheckupdate("concludeItem - deleteOrder");
		$this->orderStash->deleteOrder($params['CDFILIAL'] . '-' . $params['NRPEDIDOFOS'] . $params['NRITPEDIDOFOS']);
		$this->logcheckupdate("concludeItem - deleteOrder");
		$this->logcheckupdate("concludeItem - concludeItemStatistics");
		$this->statistics->concludeItemStatistics($params['CDFILIAL'], $params['NRPEDIDOFOS'], $params['NRITPEDIDOFOS']);
		$this->logcheckupdate("concludeItem - concludeItemStatistics");
		$this->logcheckupdate("concludeItem - updateDtUltAtuItPedidoFosByNrPedidoFos");
		$this->KDSManagerService->updateDtUltAtuItPedidoFosByNrPedidoFos($params['CDFILIAL'], $params['NRPEDIDOFOS']);
		$this->logcheckupdate("concludeItem - updateDtUltAtuItPedidoFosByNrPedidoFos", true);
	}

	private function expediteOrder($params) {
		$this->logcheckupdate("expediteOrder - doExpedition");
		$this->KDSManagerService->doExpedition($params['CDFILIAL'], $params['CDSETOR'], $params['itemsArray']);
		$this->logcheckupdate("expediteOrder - doExpedition");
		$this->logcheckupdate("expediteOrder - deleteOrder");
		$this->orderStash->deleteOrder($params['CDFILIAL'] . '-' . $params['itemsArray'][0]['NRPEDIDOFOS'] . $params['itemsArray'][0]['NRITPEDIDOFOS']);
		$this->logcheckupdate("expediteOrder - deleteOrder");
		$this->logcheckupdate("expediteOrder - expediteOrderStatistics");
		$this->statistics->expediteOrderStatistics($params['CDFILIAL'], $params['CDSETOR'], $params['itemsArray']);
		$this->logcheckupdate("expediteOrder - expediteOrderStatistics");
		$this->logcheckupdate("expediteOrder - updateDtUltAtuItPedidoFosByNrPedidoFos");
		$this->KDSManagerService->updateDtUltAtuItPedidoFosByNrPedidoFos($params['CDFILIAL'], $params['itemsArray'][0]['NRPEDIDOFOS']);
		$this->logcheckupdate("expediteOrder - updateDtUltAtuItPedidoFosByNrPedidoFos", true);
	}

	private function concludeZoomEvent($params) {
		$this->KDSManagerService->doZoomEvent($params['CDFILIAL'], $params['NRPEDIDOFOS'], $params['NRITPEDIDOFOS']);
	}

	private function releaseOrder($params) {
		$this->OrdersEngine->releaseOrder($params);
	}

	private function cancelItem($params) {
		$this->OrdersEngine->cancelItem($params);
	}

	private function deleteMessage($params) {
		// um dia isso vai ser um socket amem
		$this->messageService->sendDeletedMessagesToBrowser(array($params['CDMENSAGEM']));
	}

	public function getDataFromOneClient($params, $socketClient) {

		$this->statistics->deleteOldStatistics();
		$this->KDSManagerController->getDataFromOneClient($params['CDFILIAL'], $params['CDSETOR'], $params['IDTIPOSETOR'], $socketClient);
	    $this->messageService->getAndSendOldMessages($params['CDSETOR'], $socketClient);
	}

	private function clearRequestsScreen($params) {
		$this->KDSManagerService->clearRequestsScreen($params['CDFILIAL'], $params['CDSETOR'], $params['NRHORA']);
	}

	private function updateTransferTable($params) {
		$this->KDSTransferService->updateTransferTable($params['statusMesa'], $params['NRPRODCOMVEN'], $params['itemComanda'], $params['mesaDestino'], $params['mesaOrigem']);
	}

	private function updateNotificationTime($params) {
		$this->KDSManagerService->updateNotificationTime($params['CDFILIAL'], $params['NRPEDIDOFOS']);
	}

	private function refreshOrderItem($params) {
		$this->KDSManagerService->refreshOrderItem($params['NRVENDAREST']);
	}
}