var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive("zhColorPicker", function($timeout) {
	  return {
	  	restrict: 'A',
		  link: function($scope, $element) {

		  	var widget = $scope.widget;

		  	var hasOldColor = !!widget.oldColor;

		  	$scope.oldHsv = widget.oldColor || {h: 0, s: 0, v: 100};

		  	var $oldColor = $element.find(".old-color");
		  	var $currentColor = $element.find(".current-color");

		  	var $brightness = $element.find(".brightness-slider");
		  	var $brightnessPointer = $element.find(".brightness-pointer");

		  	var $wheel = $element.find(".color-wheel");
		  	var $wheelPointer = $element.find(".wheel-pointer");

		  	$wheel.radius = function() {
		  		return (this.width() / 2);
		  	};

		  	$wheelPointer.top = function() {
		  		return parseInt(this.css('top'));
		  	};

		  	$wheelPointer.left = function() {
		  		return parseInt(this.css('left'));
		  	};

		  	$brightnessPointer.top = function() {
		  		return parseInt(this.css('top'));
		  	};

		  	/**
				 * Wheel pointer dimmensions: 36x50 px
				 * Brightness pointer dimmensions: 50x36 px
		  	 */

		  	var WHEEL_POINTER_WIDTH = 36;
		  	var WHEEL_POINTER_HEIGHT = 50;
		  	var BRIGHTNESS_POINTER_WIDTH = 50;
		  	var BRIGHTNESS_POINTER_HEIGHT = 36;

		  	/**
				 * Translates the position of the wheel-pointer that was originally related from its
				 * own top-left point and it's parent top-left point to a more apropriate reference
				 * i.e. its bottom-center point (the inferior edge of the wheel pointer) and
				 * its parent center (the origin point of the color wheel).
				 *
				 * It's worth noting that the visual part of this, i.e. translating the picker
				 * top-left point to the inferior edge of the wheel picker is done via CSS margins.
  			 */

  			function translateXWheelPosition(wheelPointerXPosition) {
  				return wheelPointerXPosition - $wheel.radius();
  			}

  			function translateYWheelPosition(wheelPointerYPosition) {
  				return $wheel.radius() - wheelPointerYPosition;
  			}

  			function untranslateXWheelPosition(wheelPointerXPosition) {
  				return wheelPointerXPosition + $wheel.radius();
  			}

  			function untranslateYWheelPosition(wheelPointerYPosition) {
  				return $wheel.radius() + wheelPointerYPosition;
  			}

  			/**
				 * Translates the position of the brightness-pointer that was originally related from its
				 * own top-left point and its parent top-left point to a more apropriate reference
				 * i.e. its middle-right point (the right edge of the brightness pointer) and its
				 * parent top-center point (the topmost point of the parent's radial border).
				 *
				 * It's worth noting that the the visual part of this translation, i.e. translating the picker
				 * top-left point to the middle-right edge of the brightness picker could not be done with CSS margins
				 * which leads us to calculate the proper top position of the brightness picker in the JS layer.
  			 */

		  	function translateYBrightnessPosition(brightnessPointerYPosition) {
		  		return Math.abs( (brightnessPointerYPosition + (BRIGHTNESS_POINTER_HEIGHT / 2) ) - $brightness.height() );
		  	}

		  	/**
		  	 * Checks if the current wheel pointer is positioned whithin the wheel circle.
		  	 */

  			function validWheelPointerPosition(posx, posy) {
  				return Math.sqrt( Math.pow(posx, 2) + Math.pow(posy, 2) ) <= $wheel.radius();
  			}

  			/**
				 * When the wheel pointer is driven by the user to outside the wheel
				 * it should be repositioned to the edge of the circle.
				 * This function returns the position of the pointer, restrained by the wheel circle.
  			 */

  			function getRestrainedWheelPointerPosition(posx, posy) {
  				var angle = Math.atan2(posy, posx);
  				return {
  					x: Math.floor($wheel.radius() * Math.cos(angle)),
  					y: Math.floor($wheel.radius() * Math.sin(angle))
  				};
  			}

  			/**
				 * In this color picker we used the HSV and HSL color systems, and all the math done in this
				 * directive strictly follows these systems. While the wheel and brightness selectors match
				 * the HSV standard, the generated color output for the DOM is in HSL standard, once that HSV
				 * color system is not supported in CSS currently.
				 *
				 * We really tried to make the math behind this color picker as intuitive as we could, but we still strongly
				 * recomend you take a look at the HSV and HSL color system to fully comprehend it.
  			 */

  			function getWheelColorFromPosition(posx, posy) {
					var hsv = {
						h: ( Math.round( Math.atan2(posy, posx) * (180 / Math.PI) ) + 360) % 360,
						s: Math.round( ( Math.sqrt( Math.pow(posx, 2) + Math.pow(posy, 2) ) / $wheel.radius() ) * 100),
						v: 1
					};

					// WORKAROUND: The tinycolor library has a bug when dealing with saturations in ]0,1] interval
					if(hsv.s <= 1) {
						hsv.s = 0;
					}

					return hsv;
				}

		  	function getBrightnessFromPosition(posy) {
		  		return posy / $brightness.height();
		  	}

		  	/* Component color setting functions */

		  	function setWheelPointerColor(hsv) {
		  		hsv = Util.clone(hsv);
		  		hsv.v = 1;
		  		var hsl = tinycolor(hsv).toHslString();
		  		$wheelPointer.css('color', hsl); // The SVG uses the currentColor property to avoid browser incompatibilty when setting the ellipsis fill property
		  	}

		  	function setBrightnessPointerColor(hsv) {
		  		var hsl = tinycolor(hsv).toHslString();
		  		$brightnessPointer.css('color', hsl); // The SVG uses the currentColor property to avoid browser incompatibilty when setting the ellipsis fill property
		  	}

		  	function setBrightnessBackground(hsv) {
		  		hsv = Util.clone(hsv);
		  		hsv.v = 1;
		  		var hsl = tinycolor(hsv).toHslString();
		  		var gradient = "linear-gradient(" + hsl + "0%, black 100%)";
		  		$brightness.css("background", gradient);
		  	}

				function setCurrentColorIndicatorBackground(hsv) {
		  		var hsl = tinycolor(hsv).toHslString();
		  		$currentColor.css('background', hsl);
		  	}

		  	function setOldColorIndicatorBackground(hsv) {
		  		var hsl = tinycolor(hsv).toHslString();
		  		$oldColor.css('background', hsl);
		  	}

		  	/* Color manipulation functions */

		  	function setBrightnessFromPosition(hsv) {
		  		var posy = parseInt($brightnessPointer.css("top"));
		  		posy = translateYBrightnessPosition(posy);
		  		hsv.v = getBrightnessFromPosition(posy);
		  	}

		  	function setColor(color) {
		  		$timeout(function() {
			  		$scope.hsv = tinycolor(color).toHsv();

			  		var wheelPosX = ($scope.hsv.s * $wheel.radius()) * Math.cos((360 - $scope.hsv.h) * Math.PI / 180);
			  		var wheelPosY = ($scope.hsv.s * $wheel.radius()) * Math.sin((360 - $scope.hsv.h) * Math.PI / 180);
			  		var brightnessPosY = (1 - $scope.hsv.v) * $brightness.height();

			  		wheelPosX = untranslateXWheelPosition(wheelPosX);
		  			wheelPosY = untranslateYWheelPosition(wheelPosY);

			  		setWheelPointerColor($scope.hsv);
			  		setWheelPointerPosition(wheelPosX, wheelPosY);
			  		setBightnessPointerPosition(brightnessPosY - 18);
		  		});
		  	}

		  	$scope.setColor = setColor;

		  	/* Pointer positioning functions */

		  	function setWheelPointerPosition(posx, posy) {
		  		$wheelPointer.css("top", posy);
		  		$wheelPointer.css("left", posx);
		  	}

		  	function setBightnessPointerPosition(posy) {
		  		$brightnessPointer.css("top", posy);
		  	}

		  	/* Draggable event handlers */

		  	$wheelPointer.pep({
		  		start: function(event, ui) {
		  			$wheelPointer.attr("class","wheel-pointer moving"); // Can't use addClass() method due to JQuery class manipulation issues on SVG elements
		  		},

		  		drag: function(event, obj) {
		  			var xPosition = $wheelPointer.left();
		  			var yPosition = $wheelPointer.top();

		  			var posx = translateXWheelPosition(xPosition);
				    var posy = translateYWheelPosition(yPosition);

				    if(!validWheelPointerPosition(posx, posy)) {
				    	var restrainedPosition = getRestrainedWheelPointerPosition(posx, posy);
				    	posx = restrainedPosition.x;
				    	posy = restrainedPosition.y;
				    	xPosition = untranslateXWheelPosition(posx); // We need to translate back the x axis
				    	yPosition = translateYWheelPosition(posy); // The y axis just needs to be retranslated acording to the restrained position
				    	setWheelPointerPosition(xPosition, yPosition);
				    }

				    $timeout(function() {
					    $scope.hsv = getWheelColorFromPosition(posx, posy);
					    setWheelPointerColor($scope.hsv);
				    });
				  },

				  stop: function(event, ui) {
		  			$wheelPointer.attr("class","wheel-pointer"); // Can't use removeClass() method due to JQuery class manipulation issues on SVG elements
		  		},

		  		useCSSTranslation: false,
		  		disableSelect: false,
		  		removeMargins: false,
		  		shouldEase: false,
		  		place: false
		  	});

		  	$brightnessPointer.pep({
		  		axis: "y",
		  		start: function(event, ui) {
		  			$brightnessPointer.attr("class","brightness-pointer moving"); // Can't use addClass() method due to JQuery class manipulation issues on SVG elements
		  		},

		  		drag: function(event, obj) {
		  			var yPosition = $brightnessPointer.top();

		  			var POINTER_NEGATIVE_MARGIN = (-1) * $brightnessPointer.height() / 2;

		  			if(yPosition < POINTER_NEGATIVE_MARGIN) {
		  				yPosition = POINTER_NEGATIVE_MARGIN;
		  				setBightnessPointerPosition(yPosition);
		  			}

		  			if(yPosition > $brightness.height() + POINTER_NEGATIVE_MARGIN) {
		  				yPosition = $brightness.height() + POINTER_NEGATIVE_MARGIN;
		  				setBightnessPointerPosition(yPosition);
		  			}

		  			$timeout(function() {
		  				var posy = translateYBrightnessPosition(yPosition);
		  				$scope.hsv.v = getBrightnessFromPosition(posy);
		  			});
				  },

				  stop: function(event, ui) {
		  			$brightnessPointer.attr("class","brightness-pointer"); // Can't use removeClass() method due to JQuery class manipulation issues on SVG elements
		  		},

		  		useCSSTranslation: false,
		  		disableSelect: false,
		  		removeMargins: false,
		  		shouldEase: false,
		  		place: false
		  	});

		  	$scope.$watch("hsv",function() {
		  		if ($scope.hsv) {
		  			setBrightnessBackground($scope.hsv);
		  			setBrightnessFromPosition($scope.hsv);
		  			setBrightnessPointerColor($scope.hsv);

		  			setCurrentColorIndicatorBackground($scope.hsv);

		  			if(!hasOldColor) {
			  			setOldColorIndicatorBackground($scope.hsv);
		  			}

		  			widget.setCurrentRow($scope.hsv);
		  		}
	  		}, true);

		  	/* Initial function calls */
		  	setColor($scope.oldHsv);
	  		setOldColorIndicatorBackground($scope.oldHsv);

		  }
	  };
});
